use strict
use warnings;
package Devel::Debug::Server::Server;

# PODNAME: server module

# ABSTRACT: the server module 

1;

__END__

=pod

=head1 NAME

server module - the server module 

=head1 VERSION

version 0.007

=head1 AUTHOR

Jean-Christian HASSLER <hasslerjeanchristian@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Jean-Christian HASSLER.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
